// =========================================
// SCREENSAVER FRAMEWORK FOR TMT PASCAL 4.01
// -----------------------------------------
//
// Author: Georgy Moshkin
// ======================
//
// This source code is Public Domain. Feel free to use or enhance this code.
// You can use the program for any purpose you see fit. This source code is freely
// distributable without licensing fees.
//
// Based on miscellaneous Public Domain source code.


// ------------
// Fast HOW-TO:
// ------------
// 1) go to the end of the file and find this: XINST('itworks.exe','itworks.scr');
// 2) change "itworks" to name your prefer
// 3) find PROCEDURE MyRender(Window: hWnd);
// 4) add your code to MyRender procedure body
// 5) that's all!
//
// Tip:  change StringPos ('exe', lowercase (paramstr(0)), 0)=0 then
//           to StringPos ('zzz'
//           or something like that
//           for debugging purposes

//\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\\
// WWW: http://www.cnt.ru/users/coding/df/  \\
// e-mail: tmtlib@narod.ru                  \\
//\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\\

program itworks; // IT WORK'S _ i think :)

uses
  Messages, // - for working with MS Windows messages (WM_... constants)
  Windows,  // - MS Windows API routines (MessageBox procedure, ...)
  Strings,  // - contains strPCopy procedure
  OpenGL,   // - OpenGL API routines
  strman,   // - working with strings
  inst,     // - installation procedures
  ShellAPI; // - contains ShellExecute procedure

type

  // --------------
  // Running modes:
  // --------------
  // ssSetPwd - screensaver password window
  // ssPreview - run in small window (Microsoft Windows desktop properties preview mode)
  // ssConfig - screensaver configuration dialog
  // ssRun - run in fullscreen mode

  TRunningMode = (ssSetPwd, ssPreview, ssConfig, ssRun);

  // ---------------------
  // How MS Windows works?
  // ---------------------
  // MS Windows runs screensaver with the following command-line parameters:
  //
  // itworks.exe A
  // itworks.exe P 12345
  // itworks.exe C
  // itworks.exe


  // ------------------------------------
  // Meanings of command-line parameters:
  // ------------------------------------
  //
  // A - show your screensaver password window
  // P 12345 - run your screensaver in preview mode (note: MS Windows already created small
  //                                                 preview window with some handle. For example,
  //                                                 window handle is 12345)
  // C - show your screensaver configuration dialog
  //
  // And the last one without any parameters - run your screensaver in fullscreen mode
  // In this case we must create window (and get window handle) by ourselves


  // -----------------------------------------
  // How to make Built-in installation system?
  // -----------------------------------------
  // Just doing the following trick:
  // If filename ends with '.exe' then we run Installation routines
  // If filename ends with '.scr' then we run Screensaver routines
  //
  // Installation routines:
  // 1) copy screensaver file to MS Windows SYSTEM directory
  // 2) rename OURSCREENSAVER.EXE to OURSCREENSAVER.SCR


  // ------------------------------------------------
  // Initialisation procedure parameters description:
  // ------------------------------------------------
  // Window - window handle (if mode is -ssRun- then we create this handle by ourselves
  //                         if mode is -ssPreview- then we parse command-line parameters)
  // Mode - running mode (ssSetPwd, ssPreview, ssConfig or ssRun)

  TInitProc = procedure (Window: hWnd; Mode: TRunningMode);


  // -----------------------------------------------
  // Multi-purpose procedure parameters description:
  // -----------------------------------------------
  // Window - window handle

  TWindowProc = procedure(Window: hWnd);


const TimerID = 1;   // Identificator of timer
      Threshold = 5; // Threshold of mouse cursor position in pixels
                     // (exit screensaver when exceed)

var
  RunningMode: TRunningMode; // store mode depending on command line parameters
  MousePoint: TPoint;        // mouse coordinates

  InitProc: TInitProc;       // initialistaion procedure
  RenderProc: TWindowProc;   // rendering procedure
  CloseProc: TWindowProc;    // exit procede (close window, free resources)

  TimerRunning: Boolean;     // is timer running? true/false
  TimerPeriod: Integer;      // timer period (time between two WM_TIMER messages)


  Buff1: array[0..MAX_PATH] of Char; // need this variables for strPCopy procedure
  Buff2: array[0..MAX_PATH] of Char; // need this variables for strPCopy procedure
  Buff3: array[0..MAX_PATH] of Char; // need this variables for strPCopy procedure

  QuitMyScreenSaver:boolean=false; // Quit? true/false


// -------------------------------
// This procedure allocates TIMER:
// -------------------------------
// Timer sends WM_TIMER message each ... milliseconds. We use timer in the
// following way: if we get WM_TIMER then call rendering procedure.
// It means that we call Rendering Procedure each ... millisecnds.
// (note: ... milliseconds is TimerPeriod milliseconds)

procedure StartTimer(Wnd: hWnd);
begin
  TimerRunning:= Assigned(RenderProc) and (TimerPeriod > 0) and
     (SetTimer(Wnd, TimerID, TimerPeriod, nil) <> 0);
end;

// -----------------------------
// This procedure deletes TIMER:
// -----------------------------
// Also we call PeekMessage few times to make sure
// that all messages from deleted timer are removed.

procedure StopTimer(Wnd: hWnd);
var
  Msg: TMsg;
begin
  if TimerRunning then
  begin
    KillTimer(Wnd, TimerID);
    repeat until not PeekMessage( Msg, Wnd, WM_TIMER, WM_TIMER, PM_REMOVE);
    TimerRunning:= false
  end;
end;


////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

function DoScreenSaver(
    myInitProc   : TInitProc;
    myRenderProc : TWindowProc;
    myCloseProc  : TWindowProc;
    myConfigProc : TWindowProc;
    myRunPeriod  : Integer): TRunningMode;


  function WndProc(Wnd: HWND; Msg: UINT; wParam: WPARAM; lParam: LPARAM): DWORD; stdcall;
  var
    Pt: TPoint;
  begin
    Result:= 1;

    case Msg of
      WM_CREATE:
      begin
        Result:= 0;
        if Assigned(InitProc) then
          InitProc(Wnd, RunningMode);
      end;

      WM_DESTROY:
      begin
        Result:= 0;
        StopTimer(Wnd);
        if Assigned(CloseProc) then
           CloseProc(Wnd);
        PostQuitMessage(0)
      end;

      WM_TIMER:
      begin
        if (wParam = TimerID) and
           Assigned(RenderProc) then
        begin
          Result:= 0;
          RenderProc(Wnd)
        end
      end;
    end;

    if (RunningMode = ssRun) then
    case Msg of

      WM_SHOWWINDOW:
      begin
        if BOOL(wParam) then
          SetCursor(0)
      end;

      WM_CLOSE:
      begin
       if (Win32Platform = VER_PLATFORM_WIN32_WINDOWS) and
         not false then
          begin
           GetCursorPos(MousePoint);
           Result:= 0
          end
      end;

      WM_SYSCOMMAND:
      begin
        if (wParam = SC_SCREENSAVE) or (wParam=SC_CLOSE) then
          Result:= 0
      end;

      WM_SETCURSOR:
      begin
        Result:= 0;
        SetCursor(0)
      end;

      WM_NCACTIVATE:
      begin
        if wParam = 0 then
          Result:= 0
      end;

      WM_ACTIVATE:
      begin
        if wParam <> 0 then
        begin
          Result:= 0;
          GetCursorPos(MousePoint)
        end;
      end;

      WM_MOUSEMOVE:
      begin
        GetCursorPos(Pt);
        if (abs(MousePoint.X - Pt.X) > Threshold) or
           (abs(MousePoint.Y - Pt.Y) > Threshold) then
        begin
          Result:= 0;
          PostMessage (Wnd, WM_CLOSE, 0, 0)
        end
      end;

      WM_LBUTTONDOWN, WM_MBUTTONDOWN,
      WM_RBUTTONDOWN, WM_KEYDOWN, WM_SYSKEYDOWN:
      begin
        Result:= 0;
        PostMessage (Wnd, WM_CLOSE, 0, 0)
      end;
    end;

   if Result = 1 then
   Result:= DefWindowProc (Wnd, msg, wParam, lParam)
  end;


  ////
  ////

  function CreateMainWindow(hParent: HWND): HWnd;
  var
    nCx, nCy: Integer;
    WndClass: TWndClass;
    rcParent: TRect;
    uStyle, uExStyle: DWORD;
    hOther: hWnd;
    pszWindowTitle: String;

    const
      pszWindowClass = 'WindowsScreenSaverClass';

  begin
    FillChar(WndClass, SizeOf(WndClass), 0);
    with WndClass do
    begin
      hCursor        := 0;
      hIcon          := LoadIcon(0, IDI_WINLOGO); //*** LoadIcon( System.MainInstance, MAKEINTATOM( IDI_APP ) );
      lpszMenuName   := nil;
      lpszClassName  := pszWindowClass;
      hbrBackground  := GetStockObject( BLACK_BRUSH );
      hInstance      := System.MainInstance;
      style          := CS_VREDRAW or CS_HREDRAW or CS_DBLCLKS or CS_OWNDC;
      lpfnWndProc    := @WndProc;
      cbWndExtra     := 0;
      cbClsExtra     := 0
    end;
    if hParent <> 0 then
     begin
      GetClientRect( hParent, rcParent );
      nCx:= rcParent.right;
      nCy:= rcParent.bottom;
      uStyle:= WS_CHILD or WS_VISIBLE or WS_CLIPCHILDREN;
      uExStyle:= 0;
      pszWindowTitle:= 'Preview'
    end else
    begin
      nCx:= GetSystemMetrics( SM_CXSCREEN );
      nCy:= GetSystemMetrics( SM_CYSCREEN );
      uStyle:= WS_POPUP or WS_VISIBLE or WS_CLIPCHILDREN or WS_CLIPSIBLINGS;

      uExStyle:= 0;
      uExStyle:= WS_EX_TOPMOST;

      pszWindowTitle:= 'Screen Saver';
      hOther:= FindWindow( pszWindowClass, strpcopy(Buff1,pszWindowTitle) );
      if (hOther <> 0) and IsWindow( hOther ) then
      begin
        SetForegroundWindow( hOther );
        Result:= 0;
        Exit
      end
    end;

    if Windows.RegisterClass( WndClass ) <> 0 then
      Result:= CreateWindowEx( uExStyle, strpcopy(Buff2,pszWindowClass), strpcopy(Buff3,pszWindowTitle),
                               uStyle, 0, 0, nCx, nCy, hParent, 0,
                               System.MainInstance, nil )
    else
     Result:= 0
end;

function TestArg1(C: Char): Boolean;
var
  S: String;
begin
  S:= UpperCase(ParamStr(1));
  case Length(S) of
    0: Result:= not true;
    1: Result:= S[1] = C;
  else
    Result:= (S[1] in ['-','/','\']) and
             (S[2] = C)
  end;
end;

var
  Wnd         : HWnd;
  SysDir      : String;
  NewLen      : Integer;
  Dll         : hModule;
  PwdFunc: function(a: LPCTSTR; b: HWND; c: DWORD; d: Pointer): DWORD; stdcall;
  Msg: TMsg;

begin


  if TestArg1('C') then
    RunningMode:= ssConfig
  else
  if TestArg1('A') then
    RunningMode:= ssSetPwd
  else
  if TestArg1('P') then
    RunningMode:= ssPreview
  else
    RunningMode:= ssRun;
  Result:= RunningMode;

  if (RunningMode = ssSetPwd) and (Win32Platform = VER_PLATFORM_WIN32_WINDOWS) then
  begin
   //...
  end else
  if RunningMode = ssConfig then
  begin
    if Assigned(myConfigProc) then
      myConfigProc(GetForegroundWindow)
  end else
  begin
    RenderProc := myRenderProc;
    InitProc   := myInitProc;
    CloseProc  := myCloseProc;
    TimerPeriod:= myRunPeriod;


      if RunningMode = ssPreview then
        Wnd:= StrToInt(ParamStr(2))
      else
        Wnd:= 0;
      Wnd:= CreateMainWindow(Wnd);
      if Wnd <> 0 then
      begin
        StartTimer(Wnd);
        if  RunningMode <> ssPreview  then
          SetForegroundWindow( Wnd );
        while GetMessage( msg, 0, 0, 0 ) do
        begin
          TranslateMessage( msg );
          DispatchMessage( msg )
        end
      end
    end;
end;

///////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////
procedure SetDCPixelFormat(DC: HDC);
var
  pfd: TPixelFormatDescriptor;
  nPixelFormat: Integer;
begin

  FillChar(pfd, SizeOf(pfd), 0);
  with pfd do begin
    nSize     := sizeof(pfd);                               // Size of this structure
    nVersion  := 1;                                         // Version number
    dwFlags   := PFD_DRAW_TO_WINDOW or PFD_SUPPORT_OPENGL
                  or PFD_DOUBLEBUFFER;
    iPixelType:= PFD_TYPE_RGBA;                             // RGBA pixel values
    cColorBits:= 24;                                        // 24-bit color
    cDepthBits:= 32;                                        // 32-bit depth buffer
    iLayerType:= PFD_MAIN_PLANE;                            // Layer type
  end;
  nPixelFormat := ChoosePixelFormat(DC, @pfd);
  if not SetPixelFormat(DC, nPixelFormat, @pfd) then exit

end;


///////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////
procedure MyInitialise(Window: hWnd; Mode: TRunningMode);

var
  FruH, FruW: Integer;
  R: TRect;
  W, H: Integer;
  RC: HGLRC;
  DC: HDC;

  dummy:dword;

 i,j:integer;

begin

    DC:= GetDC(Window);
    SetDCPixelFormat(DC);
    RC:= wglCreateContext(DC);
    wglMakeCurrent(DC, RC);
    glClearColor(0.0,0.0,0.0,1);
    glShadeModel(GL_SMOOTH);
    GetWindowRect(Window, R);
    W:= R.Right - R.Left;
    H:= R.Bottom - R.Top;
    glViewport(0, 0, W, H);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity;
    FruW:= 950;
    FruH:= FruW*H div W;
    glFrustum(-FruW, FruW, -FruH, FruH, 2000, 20000);

    if (H=0) then H:=1; // Fix: W/0 -> W/1

  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  gluPerspective(45.0,W/H,0.1,100);
  glMatrixMode(GL_MODELVIEW);

  randomize;

end;

///////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////
procedure MyQuit(Window: hWnd);
var
  FRC: HGLRC;
  DC: hDC;
begin

  QuitMyScreenSaver:=true;

  FRC:= wglGetCurrentContext;
  DC:= wglGetCurrentDC;
  wglMakeCurrent(0, 0);
  ReleaseDC(Window, DC);
  wglDeleteContext(FRC)
end;

var   LastTicks:DWORD = 0;

///////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////
procedure MyRender(Window: hWnd);
var i:integer;
begin

 glClear(GL_DEPTH_BUFFER_BIT or GL_COLOR_BUFFER_BIT);
 glLoadIdentity();

 glDisable(GL_TEXTURE_2D);
 glDisable(GL_DEPTH_TEST);

 glBlendFunc(GL_SRC_ALPHA, GL_ONE);
 glEnable(GL_BLEND);

 glTranslatef(0,0,-10);

 glRotatef((gettickcount/1000),0,0,1);
 for i:=0 to 10 do
  begin
   glColor4f(1-abs(cos(gettickcount/500+i*100)),1/(i+1),1-abs(cos(gettickcount/1000+i*100)),0.3);
    glRotatef(i*(360)*3.14/180+sin(gettickcount/100+i*15),0,0,1);
    glBegin(GL_QUADS);
    glVertex3f(0,0,0);
    glVertex3f(10,0,0);
    glVertex3f(10,10,0);
    glVertex3f(0,10,0);
   glEnd;
  end;

 glFlush;
 SwapBuffers(wglGetCurrentDC);

 sleep(1);

end;

///////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////
procedure MySetup(Wnd: hWnd);
begin
 ShellExecute(0, nil,'http://www.cnt.ru/~coding/df/',nil,nil,1);
 MessageBox(0, 'No options. Please visit http://www.cnt.ru/~coding/df/', 'Screensaver by Georgy Moshkin - SCREENSAVER SETUP 1.0', MB_OK or MB_ICONINFORMATION);
end;

begin

  if StringPos ('exe', lowercase (paramstr(0)), 0)=0 then
   DoScreenSaver(MyInitialise, MyRender, MyQuit, MySetup, 50)
  else

  if (MessageBox(0, 'Do you want to install ITWORKS screensaver?',  'Screensaver by Georgy Moshkin - INSTALL SYSTEM 1.0', MB_YESNO or MB_ICONQUESTION ) = IDYES) then
  begin
   xinst('itworks.exe','itworks.scr');
  end;


end.

//\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/
// This source code is Public Domain. Feel free to use or enhance this code.        \\
// You can use the program for any purpose you see fit. This source code is freely  \\
// distributable without licensing fees.                                            \\
//\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/